@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or
@@ -18,6 +17,7 @@
 # Authors: Quinn Storm (quinn@beryl-project.org)
 #          Patrick Niklaus (marex@opencompositing.org)
 #          Guillaume Seguin (guillaume@segu.in)
+#          Christopher Williams (christopherw@verizon.net)
 # Copyright (C) 2007 Quinn Storm
 
 import pygtk
@@ -29,6 +29,7 @@ from ccm.Constants import *
 from ccm.Conflicts import *
 from ccm.Widgets import *
 from ccm.Utils import *
+from ccm.Pages import *
 
 import locale
 import gettext
@@ -37,6 +38,8 @@ gettext.bindtextdomain("ccsm", DataDir + "/locale")
 gettext.textdomain("ccsm")
 _ = gettext.gettext
 
+NAItemText = _("N/A")
+
 class Setting(object):
 
     NoneValue = ''
@@ -59,8 +62,9 @@ class Setting(object):
         self.Reset = gtk.Button()
         if not Settings:
             self.MakeLabel()
-            Tooltips.set_tip(self.EBox, self.Setting.LongDesc)
-            Tooltips.set_tip(self.Reset,_("Reset setting to the default value"))
+            markup = "%s\n<small><i>%s</i></small>" % (self.Setting.LongDesc, self.Setting.Name)
+            self.EBox.set_tooltip_markup(markup)
+            self.Reset.set_tooltip_text(_("Reset setting to the default value"))
         self.Reset.set_image (Image (name = gtk.STOCK_CLEAR, type = ImageStock,
                                      size = gtk.ICON_SIZE_BUTTON))
         self.Reset.connect('clicked', self.DoReset)
@@ -107,12 +111,13 @@ class Setting(object):
 
         label = gtk.Label()
         desc = protect_pango_markup (self.Setting.ShortDesc)
-        style = ""
-        if self.Setting.Integrated: style = ''' foreground="blue"'''
-        label.set_markup("<span%s>%s</span>" % (style, desc))
+        style = "%s"
+        if self.Setting.Integrated:
+            style = "<i>%s</i>"
+        label.set_markup(style % desc)
         label.props.xalign = 0
         label.set_size_request(160, -1)
-        label.props.wrap_mode = gtk.WRAP_WORD
+        label.props.wrap_mode = pango.WRAP_WORD
         label.set_line_wrap(True)
         self.Label = label
 
@@ -295,6 +300,98 @@ class EnumSetting(StockSetting):
             visible = any(text in s.lower() for s in self.Info)
         return visible
 
+class RestrictedStringSetting(StockSetting):
+
+    NoneValue = ''
+
+    def _Init(self):
+        StockSetting._Init(self)
+        self.Combo = gtk.combo_box_new_text()
+        if self.List:
+            info = self.Setting.Info[1]
+        else:
+            info = self.Setting.Info
+
+        self.ItemsByName = info[0]
+        self.ItemsByValue = info[1]
+        self.SortedItems = info[2]
+
+        # Use the first item in the list as the default value
+        self.NoneValue = self.ItemsByName[self.SortedItems[0][0]]
+
+        for (i, (name, value)) in enumerate(self.SortedItems):
+            self.Combo.append_text(name)
+        self.Combo.connect('changed', self.Changed)
+
+        self.Widget = self.Combo
+        self.Box.pack_start(self.Combo, True, True)
+
+        self.OriginalValue = None
+        self.NAItemShift = 0
+
+    def _CellEdited(self, cell, path, new_text):
+        self.CurrentRow = int(path[0])
+        value = self.ItemsByName[new_text]
+        self.Store[path][self.Num] = new_text
+        self.Set(value)
+        self.Setting.Plugin.Context.Write()
+
+    def GetColumn(self, num):
+        self.Num = num
+        cell = gtk.CellRendererCombo()
+        column = gtk.TreeViewColumn(self.Setting.ShortDesc, cell, text=num)
+        model = gtk.ListStore(str)
+        for property, value in [("model", model), ("text_column", 0),
+                                ("editable", False), ("has_entry", False)]:
+            cell.set_property (property, value)
+        cell.connect("edited", self._CellEdited)
+        for item, i in self.SortedItems:
+            model.append([item])
+
+        return (str, column)
+
+    def GetItemText (self, val):
+        text = self.ItemsByValue.get(val)
+        if text is None:
+            return NAItemText
+        return self.SortedItems[text[1]][0]
+
+    def GetForRenderer(self):
+        return [self.GetItemText(val) for val in self.Setting.Value]
+
+    def _Read(self):
+        value = self.Get()
+
+        if not self.OriginalValue:
+            self.OriginalValue = value
+
+            # if current value is not provided by any restricted string extension,
+            # insert an N/A item at the beginning
+            if not self.ItemsByValue.has_key(self.OriginalValue):
+                self.NAItemShift = 1
+                self.Combo.insert_text(0, NAItemText)
+
+        if self.ItemsByValue.has_key(value):
+            self.Combo.set_active(self.ItemsByValue[self.Get()][1] + \
+                                  self.NAItemShift)
+        else:
+            self.Combo.set_active(0)
+
+    def _Changed(self):
+        active = self.Combo.get_active_text()
+        
+        if active == NAItemText:
+            activeValue = self.OriginalValue
+        else:
+            activeValue = self.ItemsByName[active]
+        self.Set(activeValue)
+
+    def _Filter(self, text, level):
+        visible = Setting._Filter(self, text, level=level)
+        if text is not None and not visible and level & FilterValue:
+            visible = any(text in s.lower() for s in self.ItemsByName)
+        return visible
+
 class BoolSetting (StockSetting):
 
     NoneValue = False
@@ -341,6 +438,7 @@ class NumberSetting(StockSetting):
         if self.Inc is None:
             self.Inc = info[2]
         inc = self.Inc
+        self.NoneValue = info[0]
         self.Adj = gtk.Adjustment(self.Get(), info[0], info[1], inc, inc*10)
         self.Spin = gtk.SpinButton(self.Adj)
 
@@ -416,6 +514,8 @@ class BaseListSetting(Setting):
     def _Init(self):
         self.Widget = gtk.VBox()
         self.EditDialog = None        
+        self.EditDialogOpen = False
+        self.PageToBeRefreshed = None
 
         self.Widgets = []
         for i, setting in enumerate(self.Settings):
@@ -567,9 +667,16 @@ class BaseListSetting(Setting):
             widget.CurrentRow = row
             widget.Read()
 
+        self.EditDialogOpen = True
         self.EditDialog.show_all()
         response = self.EditDialog.run()
         self.EditDialog.hide_all()
+        self.EditDialogOpen = False
+
+        if self.PageToBeRefreshed:
+            self.PageToBeRefreshed[0].RefreshPage(self.PageToBeRefreshed[1],
+                                                  self.PageToBeRefreshed[2])
+            self.PageToBeRefreshed = None
 
         self.Read()
 
@@ -664,7 +771,7 @@ class ListSetting(BaseListSetting):
 class MultiListSetting(BaseListSetting):
 
     def _Init(self):
-        Tooltips.set_tip(self.EBox, _("Multi-list settings. You can double-click a row to edit the values."))
+        self.EBox.set_tooltip_text(_("Multi-list settings. You can double-click a row to edit the values."))
         BaseListSetting._Init(self)
 
     def Filter(self, text, level=FilterAll):
@@ -724,6 +831,58 @@ class EnumFlagsSetting(Setting):
             visible = any(text in s.lower() for s in self.Setting.Info[1][2])
         return visible
 
+class RestrictedStringFlagsSetting(Setting):
+
+    def _Init(self):
+        frame = gtk.Frame(self.Setting.ShortDesc)
+        table = gtk.Table()
+        
+        row = col = 0
+        self.Checks = []
+        info = self.Setting.Info[1]
+        self.ItemsByName = info[0]
+        self.ItemsByValue = info[1]
+        sortedItems = info[2]
+        for key, value in sortedItems:
+            box = gtk.CheckButton(key)
+            self.Checks.append((key, box))
+            table.attach(box, col, col+1, row, row+1, TableDef, TableDef, TableX, TableX)
+            box.connect('toggled', self.Changed)
+            col = col+1
+            if col >= 3:
+                col = 0
+                row += 1
+
+        vbox = gtk.VBox()
+        vbox.pack_start(self.Reset, False, False)
+
+        hbox = gtk.HBox()
+        hbox.pack_start(table, True, True)
+        hbox.pack_start(vbox, False, False)
+
+        frame.add(hbox)
+        self.Box.pack_start(frame, True, True)
+
+    def _Read(self):
+        for key, box in self.Checks:
+            box.set_active(False)
+        for setVal in self.Setting.Value:
+            if self.ItemsByValue.has_key(setVal):
+                self.Checks[self.ItemsByValue[setVal][1]][1].set_active(True)
+
+    def _Changed(self):
+        values = []
+        for key, box in self.Checks:
+            if box.get_active():
+                values.append(self.ItemsByName[key])
+        self.Setting.Value = values
+
+    def _Filter(self, text, level=FilterAll):
+        visible = Setting._Filter(self, text, level=level)
+        if text is not None and not visible and level & FilterValue:
+            visible = any(text in s.lower() for s in self.ItemsByName)
+        return visible
+
 class EditableActionSetting (StockSetting):
 
     def _Init (self, widget, action):
@@ -736,7 +895,7 @@ class EditableActionSetting (StockSetting):
         editButton = gtk.Button ()
         editButton.add (Image (name = gtk.STOCK_EDIT, type = ImageStock,
                                size = gtk.ICON_SIZE_BUTTON))
-        Tooltips.set_tip(editButton, _("Edit %s" % self.Setting.ShortDesc))
+        editButton.set_tooltip_text(_("Edit %s" % self.Setting.ShortDesc))
         editButton.connect ("clicked", self.RunEditDialog)
 
         action = ActionImage (action)
@@ -762,7 +921,7 @@ class EditableActionSetting (StockSetting):
         alignment.set_padding (10, 10, 10, 10)
         alignment.add (entry)
 
-        Tooltips.set_tip (entry, self.Setting.LongDesc)
+        entry.set_tooltip_text(self.Setting.LongDesc)
         dlg.vbox.pack_start (alignment)
         
         dlg.vbox.show_all ()
@@ -793,7 +952,7 @@ class KeySetting (EditableActionSetting):
         EditableActionSetting._Init (self, self.Button, "keyboard")
 
     def DoReset (self, widget):
-        conflict = ActionConflict (setting = self.Setting, key=self.Setting.DefaultValue)
+        conflict = KeyConflict (self.Setting, self.Setting.DefaultValue)
         if conflict.Resolve (GlobalUpdater):
             self.Setting.Reset ()
             self.Setting.Plugin.Context.Write ()
@@ -883,7 +1042,7 @@ class KeySetting (EditableActionSetting):
         active = len (self.current) \
                  and self.current.lower () not in ("disabled", "none")
         checkButton.set_active (active)
-        Tooltips.set_tip (checkButton, self.Setting.LongDesc)
+        checkButton.set_tooltip_text(self.Setting.LongDesc)
         mainBox.pack_start (checkButton)
 
         box = gtk.VBox ()
@@ -896,7 +1055,7 @@ class KeySetting (EditableActionSetting):
                 currentMods += "%s|" % mod
         currentMods.rstrip ("|")
         modifierSelector = ModifierSelector (currentMods)
-        Tooltips.set_tip (modifierSelector, self.Setting.LongDesc)
+        modifierSelector.set_tooltip_text (self.Setting.LongDesc)
         alignment = gtk.Alignment (0.5)
         alignment.add (modifierSelector)
         box.pack_start (alignment)
@@ -904,11 +1063,11 @@ class KeySetting (EditableActionSetting):
         key, mods = gtk.accelerator_parse (self.current)
         grabber = KeyGrabber (key = key, mods = mods,
                               label = _("Grab key combination"))
-        Tooltips.set_tip (grabber, self.Setting.LongDesc)
+        grabber.set_tooltip_text (self.Setting.LongDesc)
         box.pack_start (grabber)
 
         label = gtk.Label (self.current)
-        Tooltips.set_tip (label, self.Setting.LongDesc)
+        label.set_tooltip_text (self.Setting.LongDesc)
         alignment = gtk.Alignment (0.5, 0.5)
         alignment.set_padding (15, 0, 0, 0)
         alignment.add (label)
@@ -942,10 +1101,7 @@ class KeySetting (EditableActionSetting):
     def BindingEdited (self, accel):
         '''Binding edited callback'''
         # Update & save binding
-        popup = Popup (self.Widget, 
-                       _("Computing possible conflicts, please wait"))
-        conflict = ActionConflict (self.Setting, key = accel)
-        popup.destroy ()
+        conflict = KeyConflict (self.Setting, accel)
         if conflict.Resolve (GlobalUpdater):
             self.current = accel
             self.Changed ()
@@ -966,13 +1122,13 @@ class ButtonSetting (EditableActionSetting):
 
         self.Button = SizedButton (minWidth = 100)
         self.Button.connect ("clicked", self.RunButtonSelector)
-        Tooltips.set_tip (self.Button, self.Setting.LongDesc)
+        self.Button.set_tooltip_text(self.Setting.LongDesc)
         self.SetButtonLabel ()
         
         EditableActionSetting._Init (self, self.Button, "button")
 
     def DoReset (self, widget):
-        conflict = ActionConflict (setting = self.Setting, button=self.Setting.DefaultValue)
+        conflict = ButtonConflict (self.Setting, self.Setting.DefaultValue)
         if conflict.Resolve (GlobalUpdater):
             self.Setting.Reset ()
             self.Setting.Plugin.Context.Write ()
@@ -1038,7 +1194,7 @@ class ButtonSetting (EditableActionSetting):
         active = len (self.current) \
                  and self.current.lower () not in ("disabled", "none")
         checkButton.set_active (active)
-        Tooltips.set_tip (checkButton, self.Setting.LongDesc)
+        checkButton.set_tooltip_text (self.Setting.LongDesc)
         mainBox.pack_start (checkButton)
 
         box = gtk.VBox ()
@@ -1051,7 +1207,7 @@ class ButtonSetting (EditableActionSetting):
                 currentEdges += "%s|" % edge
         currentEdges.rstrip ("|")
         edgeSelector = SingleEdgeSelector (currentEdges)
-        Tooltips.set_tip (edgeSelector, self.Setting.LongDesc)
+        edgeSelector.set_tooltip_text(self.Setting.LongDesc)
         box.pack_start (edgeSelector)
 
         currentMods = ""
@@ -1060,7 +1216,7 @@ class ButtonSetting (EditableActionSetting):
                 currentMods += "%s|" % mod
         currentMods.rstrip ("|")
         modifierSelector = ModifierSelector (currentMods)
-        Tooltips.set_tip (modifierSelector, self.Setting.LongDesc)
+        modifierSelector.set_tooltip_text(self.Setting.LongDesc)
         box.pack_start (modifierSelector)
 
         buttonCombo = gtk.combo_box_new_text ()
@@ -1069,7 +1225,7 @@ class ButtonSetting (EditableActionSetting):
             if "Button%d" % i in self.current:
                 currentButton = i
                 break
-        maxButton = 9
+        maxButton = 20
         for i in range (1, maxButton + 1):
             button = "Button%d" % i
             buttonCombo.append_text (button)
@@ -1078,7 +1234,7 @@ class ButtonSetting (EditableActionSetting):
             buttonCombo.set_active (maxButton)
         else:
             buttonCombo.set_active (currentButton - 1)
-        Tooltips.set_tip (buttonCombo, self.Setting.LongDesc)
+        buttonCombo.set_tooltip_text(self.Setting.LongDesc)
         box.pack_start (buttonCombo)
 
         dlg.vbox.show_all ()
@@ -1122,10 +1278,7 @@ to set \"%s\" button to Button1 ?") % self.Setting.ShortDesc)
             response = warning.run ()
             if response != gtk.RESPONSE_YES:
                 return
-        popup = Popup (self.Widget, 
-                       _("Computing possible conflicts, please wait"))
-        conflict = ActionConflict (self.Setting, button = button)
-        popup.destroy ()
+        conflict = ButtonConflict (self.Setting, button)
         if conflict.Resolve (GlobalUpdater):
             self.current = button
             self.Changed ()
@@ -1146,13 +1299,13 @@ class EdgeSetting (EditableActionSetting):
 
         self.Button = SizedButton (minWidth = 100)
         self.Button.connect ("clicked", self.RunEdgeSelector)
-        Tooltips.set_tip (self.Button, self.Setting.LongDesc)
+        self.Button.set_tooltip_text(self.Setting.LongDesc)
         self.SetButtonLabel ()
 
         EditableActionSetting._Init (self, self.Button, "edges")
 
     def DoReset (self, widget):
-        conflict = ActionConflict (setting = self.Setting, edges=self.Setting.DefaultValue)
+        conflict = EdgeConflict (self.Setting, self.Setting.DefaultValue)
         if conflict.Resolve (GlobalUpdater):
             self.Setting.Reset ()
             self.Setting.Plugin.Context.Write ()
@@ -1199,7 +1352,7 @@ class EdgeSetting (EditableActionSetting):
         alignment.set_padding (10, 10, 10, 10)
         alignment.add (selector)
 
-        Tooltips.set_tip (selector, self.Setting.LongDesc)
+        selector.set_tooltip_text (self.Setting.LongDesc)
         dlg.vbox.pack_start (alignment)
         
         dlg.vbox.show_all ()
@@ -1213,10 +1366,7 @@ class EdgeSetting (EditableActionSetting):
 
     def EdgeEdited (self, edge):
         '''Edge edited callback'''
-        popup = Popup (self.Widget, 
-                       _("Computing possible conflicts, please wait"))
-        conflict = ActionConflict (self.Setting, edges = edge)
-        popup.destroy ()
+        conflict = EdgeConflict (self.Setting, edge)
         if conflict.Resolve (GlobalUpdater):
             self.current = edge
             self.Changed ()
@@ -1250,6 +1400,9 @@ def MakeStringSetting (setting, List=False):
             return FileStringSetting (setting, isDirectory=True, List=List)
         else:
             return StringSetting (setting, List=List)
+    elif (List and setting.Info[1][2]) or \
+        (not List and setting.Info[2]):
+        return RestrictedStringSetting (setting, List=List)
     else:
         return StringSetting (setting, List=List)
 
@@ -1272,6 +1425,8 @@ def MakeListSetting (setting, List=False):
 
     if setting.Info[0] == "Int" and setting.Info[1][2]:
         return EnumFlagsSetting (setting)
+    elif setting.Info[0] == "String" and setting.Info[1][2]:
+        return RestrictedStringFlagsSetting (setting)
     else:
         return ListSetting (setting)
       
@@ -1307,7 +1462,7 @@ class SubGroupArea(object):
         self.MySettings = []
         self.SubGroup = subGroup
         self.Name = name
-        settings = sorted(sum((v.values() for v in [subGroup.Display]+[subGroup.Screens[CurrentScreenNum]]), []), key=SettingKeyFunc)
+        settings = sorted(GetSettings(subGroup), key=SettingKeyFunc)
         if not name:
             self.Child = self.Widget = gtk.VBox()
         else:
