@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or
@@ -923,6 +922,7 @@ class Popup (gtk.Window):
             self.set_transient_for (parent.get_toplevel ())
         self.set_modal (modal)
         self.set_decorated (decorated)
+        self.set_property("skip-taskbar-hint", True)
         if text:
             label = gtk.Label (text)
             align = gtk.Alignment ()
@@ -1124,6 +1124,13 @@ class MatchButton(gtk.Button):
 
         self.set_match (match)
 
+    def _check_entry_value (self, entry, dialog):
+        is_valid = False
+        value = entry.get_text()
+        if value != "":
+            is_valid = True
+        dialog.set_response_sensitive(gtk.RESPONSE_OK, is_valid)
+
     def run_edit_dialog (self, widget):
         '''Run dialog to generate a match'''
 
@@ -1134,6 +1141,7 @@ class MatchButton(gtk.Button):
         dlg.set_transient_for (self.get_parent ().get_toplevel ())
         dlg.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
         dlg.add_button (gtk.STOCK_ADD, gtk.RESPONSE_OK).grab_default ()
+        dlg.set_response_sensitive(gtk.RESPONSE_OK, False)
         dlg.set_default_response (gtk.RESPONSE_OK)
 
         table = gtk.Table ()
@@ -1153,6 +1161,7 @@ class MatchButton(gtk.Button):
         box = gtk.HBox ()
         box.set_spacing (5)
         entry = gtk.Entry ()
+        entry.connect ('changed', self._check_entry_value, dlg)
         button = gtk.Button (_("Grab"))
         button.connect ('clicked', self.grab_value, entry, type_chooser)
         box.pack_start (entry, True, True)
@@ -1183,7 +1192,7 @@ class MatchButton(gtk.Button):
         dlg.show_all ()
 
         response = dlg.run ()
-        dlg.destroy ()
+        dlg.hide ()
         if response == gtk.RESPONSE_OK:
             type     = type_chooser.get_active_text ()
             value    = entry.get_text ()
@@ -1191,6 +1200,8 @@ class MatchButton(gtk.Button):
             invert   = check.get_active ()
             self.generate_match (type, value, relation, invert)
 
+        dlg.destroy ()
+
 class FileButton (gtk.Button):
     __gsignals__    = {"changed" : (gobject.SIGNAL_RUN_FIRST,
                                     gobject.TYPE_NONE,
@@ -1209,7 +1220,7 @@ class FileButton (gtk.Button):
         self._image = image
         self._path = path
 
-        Tooltips.set_tip(self, _("Browse..."))
+        self.set_tooltip_text(_("Browse..."))
         self.set_image(gtk.image_new_from_stock(
             gtk.STOCK_OPEN, gtk.ICON_SIZE_BUTTON))
         self.connect('clicked', self.open_dialog)
@@ -1310,7 +1321,9 @@ class AboutDialog (gtk.AboutDialog):
         self.set_artists (["Andrew Wedderburn <andrew.wedderburn@gmail.com>",
                            "Patrick Niklaus <marex@opencompositing.org>",
                            "Gnome Icon Theme Team"])
-        self.set_logo (IconTheme.load_icon("ccsm", 64, gtk.ICON_LOOKUP_FORCE_SVG))
+        if IconTheme.lookup_icon("ccsm", 64, 0):
+            icon = IconTheme.load_icon("ccsm", 64, 0)
+            self.set_logo (icon)
         self.set_website ("http://www.compiz-fusion.org")
 
 # Error dialog
@@ -1360,12 +1373,13 @@ class PluginButton (gtk.HBox):
 
     _plugin = None
 
-    def __init__ (self, plugin):
+    def __init__ (self, plugin, useMissingImage = False):
         gtk.HBox.__init__(self)
         self._plugin = plugin
 
-        image = Image (plugin.Name, ImagePlugin, size=32)
+        image = Image (plugin.Name, ImagePlugin, 32, useMissingImage)
         label = Label (plugin.ShortDesc, 120)
+        label.connect ('style-set', self.style_set)
         box = gtk.HBox ()
         box.set_spacing (5)
         box.pack_start (image, False, False)
@@ -1373,12 +1387,12 @@ class PluginButton (gtk.HBox):
 
         button = PrettyButton ()
         button.connect ('clicked', self.show_plugin_page)
-        Tooltips.set_tip (button, plugin.LongDesc)
+        button.set_tooltip_text (plugin.LongDesc)
         button.add (box)
 
         if plugin.Name != 'core':
             enable = gtk.CheckButton ()
-            Tooltips.set_tip (enable, _("Enable %s") % plugin.ShortDesc)
+            enable.set_tooltip_text(_("Enable %s") % plugin.ShortDesc)
             enable.set_active (plugin.Enabled)
             enable.set_sensitive (plugin.Context.AutoSort)
             self._toggled_handler = enable.connect ("toggled", self.enable_plugin)
@@ -1388,6 +1402,15 @@ class PluginButton (gtk.HBox):
 
         self.set_size_request (220, -1)
 
+    StyleBlock = 0
+
+    def style_set (self, widget, previous):
+        if self.StyleBlock > 0:
+            return
+        self.StyleBlock += 1
+        widget.modify_fg(gtk.STATE_NORMAL, widget.style.text[gtk.STATE_NORMAL])
+        self.StyleBlock -= 1
+
     def enable_plugin (self, widget):
 
         plugin = self._plugin
@@ -1404,6 +1427,7 @@ class PluginButton (gtk.HBox):
 
         plugin.Context.Write ()
         GlobalUpdater.UpdatePlugins()
+        plugin.Context.UpdateExtensiblePlugins ()
         self.emit ('activated')
 
     def show_plugin_page (self, widget):
@@ -1434,6 +1458,7 @@ class PluginButton (gtk.HBox):
 class CategoryBox(gtk.VBox):
 
     _plugins = None
+    _unfiltered_plugins = None
     _buttons = None
     _context = None
     _name    = ""
@@ -1442,7 +1467,7 @@ class CategoryBox(gtk.VBox):
     _current_cols = 0
     _current_plugins = 0
 
-    def __init__ (self, context, name, plugins=None):
+    def __init__ (self, context, name, plugins=None, categoryIndex=0):
         gtk.VBox.__init__ (self)
 
         self.set_spacing (5)
@@ -1462,10 +1487,14 @@ class CategoryBox(gtk.VBox):
         self._name = name
         text = name or 'Uncategorized'
 
+        # Keep unfiltered list of plugins for correct background icon loading
+        self._unfiltered_plugins = self._plugins
+
         header = gtk.HBox ()
+        header.set_border_width (5)
         header.set_spacing (10)
         label = Label ('', -1)
-        label.set_markup ("<span color='#aaa' size='x-large' weight='800'>%s</span>" % text)
+        label.set_markup ("<span color='#aaa' size='x-large' weight='800'>%s</span>" % _(text))
 
         icon = text.lower ().replace (" ", "_")
         image = Image (icon, ImageCategory)
@@ -1475,9 +1504,12 @@ class CategoryBox(gtk.VBox):
         self._table = gtk.Table ()
         self._table.set_border_width (10)
 
+        # load icons now only for the first 3 categories
+        dontLoadIcons = (categoryIndex >= 3);
+
         self._buttons = []
         for plugin in self._plugins:
-            button = PluginButton(plugin)
+            button = PluginButton(plugin, dontLoadIcons)
             self._buttons.append(button)
 
         self._alignment = gtk.Alignment (0, 0, 1, 1)
@@ -1505,8 +1537,8 @@ class CategoryBox(gtk.VBox):
 
         return bool(self._plugins)
 
-    def rebuild_table (self, ncols):
-        if (ncols == self._current_cols
+    def rebuild_table (self, ncols, force = False):
+        if (not force and ncols == self._current_cols
         and len (self._plugins) == self._current_plugins):
             return
         self._current_cols = ncols
@@ -1534,6 +1566,9 @@ class CategoryBox(gtk.VBox):
     def get_plugins (self):
         return self._plugins
 
+    def get_unfiltered_plugins (self):
+        return self._unfiltered_plugins
+
 # Plugin Window
 #
 class PluginWindow(gtk.ScrolledWindow):
@@ -1580,12 +1615,10 @@ class PluginWindow(gtk.ScrolledWindow):
         self._not_found_box = NotFoundBox ()
 
         categories = sorted(self._categories, key=CategoryKeyFunc)
-        for category in categories:
+        for (i, category) in enumerate(categories):
             plugins = self._categories[category]
-            category_box = CategoryBox(context, category, plugins)
-            buttons = category_box.get_buttons ()
-            for button in buttons:
-                button.connect('clicked', self.show_plugin_page)
+            category_box = CategoryBox(context, category, plugins, i)
+            self.connect_buttons (category_box)
             self._boxes.append (category_box)
             self._box.pack_start (category_box, False, False)
 
@@ -1595,6 +1628,11 @@ class PluginWindow(gtk.ScrolledWindow):
         viewport.add (self._box)
         self.add (viewport)
 
+    def connect_buttons (self, category_box):
+        buttons = category_box.get_buttons ()
+        for button in buttons:
+            button.connect('clicked', self.show_plugin_page)
+
     def set_viewport_style (self, widget, previous):
         if self._style_block > 0:
             return
